@@ -1,6 +1,8 @@
 #!/usr/bin/python3
 #
 # Copyright (c) 2013-2019 The Khronos Group Inc.
+# Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+# Copyright (c) 2023-2023 RasterGrid Kft.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -47,8 +49,14 @@ def makeGenOpts(args):
     global genOpts
     genOpts = {}
 
+    # API to generate sources for
+    apiname = args.api
+
     # Default class of extensions to include, or None
-    defaultExtensions = args.defaultExtensions
+    if args.defaultExtensions is not None:
+        defaultExtensions = args.defaultExtensions
+    else:
+        defaultExtensions = apiname
 
     # Additional extensions to include (list of extensions)
     extensions = args.extension
@@ -126,11 +134,11 @@ def makeGenOpts(args):
             filename          = 'vk_dispatch_table_helper.h',
             directory         = directory,
             genpath           = None,
-            apiname           = 'vulkan',
+            apiname           = apiname,
             profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
+            defaultExtensions = defaultExtensions,
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
@@ -150,11 +158,11 @@ def makeGenOpts(args):
             filename          = 'vk_layer_dispatch_table.h',
             directory         = directory,
             genpath           = None,
-            apiname           = 'vulkan',
+            apiname           = apiname,
             profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
+            defaultExtensions = defaultExtensions,
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
@@ -174,11 +182,11 @@ def makeGenOpts(args):
             filename          = 'vk_loader_extensions.h',
             directory         = directory,
             genpath           = None,
-            apiname           = 'vulkan',
+            apiname           = apiname,
             profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
+            defaultExtensions = defaultExtensions,
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
@@ -198,11 +206,11 @@ def makeGenOpts(args):
             filename          = 'vk_loader_extensions.c',
             directory         = directory,
             genpath           = None,
-            apiname           = 'vulkan',
+            apiname           = apiname,
             profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
+            defaultExtensions = defaultExtensions,
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
@@ -222,11 +230,11 @@ def makeGenOpts(args):
             filename          = 'vk_object_types.h',
             directory         = directory,
             genpath           = None,
-            apiname           = 'vulkan',
+            apiname           = apiname,
             profile           = None,
             versions          = featuresPat,
             emitversions      = featuresPat,
-            defaultExtensions = 'vulkan',
+            defaultExtensions = defaultExtensions,
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
@@ -262,6 +270,7 @@ def genTarget(args):
 
         if not args.quiet:
             write('* Building', options.filename, file=sys.stderr)
+            write('* options.apiname           =', options.apiname, file=sys.stderr)
             write('* options.versions          =', options.versions, file=sys.stderr)
             write('* options.emitversions      =', options.emitversions, file=sys.stderr)
             write('* options.defaultExtensions =', options.defaultExtensions, file=sys.stderr)
@@ -286,8 +295,12 @@ def genTarget(args):
 if __name__ == '__main__':
     parser = argparse.ArgumentParser()
 
-    parser.add_argument('-defaultExtensions', action='store',
+    parser.add_argument('-api', action='store',
                         default='vulkan',
+                        choices=['vulkan'],
+                        help='Specify API name to generate')
+    parser.add_argument('-defaultExtensions', action='store',
+                        default=None,
                         help='Specify a single class of extensions to add to targets')
     parser.add_argument('-extension', action='append',
                         default=[],
