@@ -108,11 +108,18 @@ struct TestLayer {
     BUILDER_VECTOR(TestLayer, std::string, alternative_function_names, alternative_function_name)
 
     BUILDER_VECTOR(TestLayer, Extension, instance_extensions, instance_extension)
+    std::vector<Extension> enabled_instance_extensions;
+
     BUILDER_VECTOR(TestLayer, Extension, device_extensions, device_extension)
 
     BUILDER_VALUE(TestLayer, std::string, enable_environment, {});
     BUILDER_VALUE(TestLayer, std::string, disable_environment, {});
 
+    // Modifies the extension list returned by vkEnumerateInstanceExtensionProperties to include what is in this vector
+    BUILDER_VECTOR(TestLayer, Extension, injected_instance_extensions, injected_instance_extension)
+    // Modifies the extension list returned by  vkEnumerateDeviceExtensionProperties to include what is in this vector
+    BUILDER_VECTOR(TestLayer, Extension, injected_device_extensions, injected_device_extension)
+
     BUILDER_VECTOR(TestLayer, LayerDefinition, meta_component_layers, meta_component_layer);
 
     BUILDER_VALUE(TestLayer, bool, intercept_vkEnumerateInstanceExtensionProperties, false)
@@ -160,6 +167,8 @@ struct TestLayer {
         return nullptr;
     }
 
+    // Allows distinguishing different layers (that use the same binary)
+    BUILDER_VALUE(TestLayer, std::string, make_spurious_log_in_create_instance, "")
     BUILDER_VALUE(TestLayer, bool, do_spurious_allocations_in_create_instance, false)
     void* spurious_instance_memory_allocation = nullptr;
     BUILDER_VALUE(TestLayer, bool, do_spurious_allocations_in_create_device, false)
@@ -172,6 +181,20 @@ struct TestLayer {
     // By default query GPDPA from GIPA, don't use value given from pNext
     BUILDER_VALUE(TestLayer, bool, use_gipa_GetPhysicalDeviceProcAddr, true)
 
+    // Have a layer query for vkCreateDevice with a NULL instance handle
+    BUILDER_VALUE(TestLayer, bool, buggy_query_of_vkCreateDevice, false)
+
+    // Makes the layer try to create a device using the loader provided function in the layer chain
+    BUILDER_VALUE(TestLayer, bool, call_create_device_while_create_device_is_called, false)
+    BUILDER_VALUE(TestLayer, uint32_t, physical_device_index_to_use_during_create_device, 0)
+
+    BUILDER_VALUE(TestLayer, bool, check_if_EnumDevExtProps_is_same_as_queried_function, false)
+
+    // Clober the data pointed to by pInstance to overwrite the magic value
+    BUILDER_VALUE(TestLayer, bool, clobber_pInstance, false)
+    // Clober the data pointed to by pDevice to overwrite the magic value
+    BUILDER_VALUE(TestLayer, bool, clobber_pDevice, false)
+
     PFN_vkGetInstanceProcAddr next_vkGetInstanceProcAddr = VK_NULL_HANDLE;
     PFN_GetPhysicalDeviceProcAddr next_GetPhysicalDeviceProcAddr = VK_NULL_HANDLE;
     PFN_vkGetDeviceProcAddr next_vkGetDeviceProcAddr = VK_NULL_HANDLE;
@@ -182,8 +205,15 @@ struct TestLayer {
     struct Device {
         VkDevice device_handle = VK_NULL_HANDLE;
         VkLayerDispatchTable dispatch_table{};
+        std::vector<Extension> enabled_extensions;
     };
     std::vector<Device> created_devices;
+
+    // Stores the callback that allows layers to create devices on their own
+    PFN_vkLayerCreateDevice callback_vkCreateDevice{};
+    PFN_vkLayerDestroyDevice callback_vkDestroyDevice{};
+    std::vector<VkPhysicalDevice> queried_physical_devices;
+    Device second_device_created_during_create_device{};
 };
 
 using GetTestLayerFunc = TestLayer* (*)();
