@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2021-2022 The Khronos Group Inc.
- * Copyright (c) 2021-2022 Valve Corporation
- * Copyright (c) 2021-2022 LunarG, Inc.
+ * Copyright (c) 2021-2023 The Khronos Group Inc.
+ * Copyright (c) 2021-2023 Valve Corporation
+ * Copyright (c) 2021-2023 LunarG, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and/or associated documentation files (the "Materials"), to
@@ -28,21 +28,6 @@
 
 #include "test_environment.h"
 
-class EnvVarICDOverrideSetup : public ::testing::Test {
-   protected:
-    virtual void SetUp() {
-        remove_env_var("VK_ICD_FILENAMES");
-        remove_env_var("VK_DRIVER_FILES");
-        remove_env_var("VK_ADD_DRIVER_FILES");
-    }
-
-    virtual void TearDown() {
-        remove_env_var("VK_ICD_FILENAMES");
-        remove_env_var("VK_DRIVER_FILES");
-        remove_env_var("VK_ADD_DRIVER_FILES");
-    }
-};
-
 // Don't support vk_icdNegotiateLoaderICDInterfaceVersion
 // Loader calls vk_icdGetInstanceProcAddr second
 // does not support vk_icdGetInstanceProcAddr
@@ -107,7 +92,7 @@ TEST(EnvVarICDOverrideSetup, version_2_negotiate_interface_version_and_icd_gipa_
 TEST(EnvVarICDOverrideSetup, TestOnlyDriverEnvVar) {
     FrameworkEnvironment env{};
     env.add_icd(TestICDDetails(TEST_ICD_PATH_EXPORT_NONE).set_discovery_type(ManifestDiscoveryType::env_var));
-    env.get_test_icd(0).physical_devices.emplace_back("pd0");
+    env.get_test_icd(0).add_physical_device("pd0");
 
     InstWrapper inst1{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst1.create_info, env.debug_log);
@@ -121,9 +106,9 @@ TEST(EnvVarICDOverrideSetup, TestOnlyDriverEnvVar) {
     ASSERT_EQ(phys_dev_count, 1U);
 
     for (uint32_t add = 0; add < 2; ++add) {
-        env.add_icd(TestICDDetails(TEST_ICD_PATH_EXPORT_NONE).set_discovery_type(ManifestDiscoveryType::env_var));
-        env.get_test_icd(add + 1).physical_devices.emplace_back("pd" + std::to_string(add) + "0");
-        env.get_test_icd(add + 1).physical_devices.emplace_back("pd" + std::to_string(add) + "1");
+        env.add_icd(TestICDDetails(TEST_ICD_PATH_EXPORT_NONE).set_discovery_type(ManifestDiscoveryType::env_var))
+            .add_physical_device("pd" + std::to_string(add) + "0")
+            .add_physical_device("pd" + std::to_string(add) + "1");
     }
 
     env.debug_log.clear();
@@ -149,19 +134,64 @@ TEST(EnvVarICDOverrideSetup, TestOnlyDriverEnvVar) {
     env.platform_shim->set_elevated_privilege(false);
 }
 
-#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__)
-// Make sure the loader reports the correct message based on if USE_UNSAFE_FILE_SEARCH is set or not
+// Test VK_DRIVER_FILES environment variable containing a path to a folder
+TEST(EnvVarICDOverrideSetup, TestOnlyDriverEnvVarInFolder) {
+    FrameworkEnvironment env{};
+    env.add_icd(TestICDDetails(TEST_ICD_PATH_EXPORT_NONE).set_discovery_type(ManifestDiscoveryType::env_var).set_is_dir(false));
+    env.get_test_icd(0).add_physical_device("pd0");
+
+    InstWrapper inst1{env.vulkan_functions};
+    FillDebugUtilsCreateDetails(inst1.create_info, env.debug_log);
+    inst1.CheckCreate();
+    EXPECT_FALSE(
+        env.debug_log.find("Ignoring override VK_ICD_FILENAMES, VK_DRIVER_FILES, and VK_ADD_DRIVER_FILES due to high-integrity"));
+
+    std::array<VkPhysicalDevice, 5> phys_devs_array;
+    uint32_t phys_dev_count = 1;
+    ASSERT_EQ(inst1->vkEnumeratePhysicalDevices(inst1.inst, &phys_dev_count, phys_devs_array.data()), VK_SUCCESS);
+    ASSERT_EQ(phys_dev_count, 1U);
+
+    for (uint32_t add = 0; add < 2; ++add) {
+        env.add_icd(TestICDDetails(TEST_ICD_PATH_EXPORT_NONE).set_discovery_type(ManifestDiscoveryType::env_var))
+            .add_physical_device("pd" + std::to_string(add) + "0")
+            .add_physical_device("pd" + std::to_string(add) + "1");
+    }
+
+    env.debug_log.clear();
+
+    InstWrapper inst2{env.vulkan_functions};
+    FillDebugUtilsCreateDetails(inst2.create_info, env.debug_log);
+    inst2.CheckCreate();
+
+    phys_dev_count = 5;
+    ASSERT_EQ(inst2->vkEnumeratePhysicalDevices(inst2.inst, &phys_dev_count, phys_devs_array.data()), VK_SUCCESS);
+    ASSERT_EQ(phys_dev_count, 5U);
+
+    env.debug_log.clear();
+
+    env.platform_shim->set_elevated_privilege(true);
+
+    InstWrapper inst3{env.vulkan_functions};
+    FillDebugUtilsCreateDetails(inst3.create_info, env.debug_log);
+    inst3.CheckCreate(VK_ERROR_INCOMPATIBLE_DRIVER);
+
+    EXPECT_TRUE(env.debug_log.find("vkCreateInstance: Found no drivers!"));
+
+    env.platform_shim->set_elevated_privilege(false);
+}
+
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__GNU__)
+// Make sure the loader reports the correct message based on if LOADER_USE_UNSAFE_FILE_SEARCH is set or not
 TEST(EnvVarICDOverrideSetup, NonSecureEnvVarLookup) {
     FrameworkEnvironment env{};
-    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA));
-    env.get_test_icd().physical_devices.emplace_back("physical_device_0");
+    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA)).add_physical_device("physical_device_0");
 
     DebugUtilsLogger log{VK_DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT};
     InstWrapper inst{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst.create_info, log);
     inst.CheckCreate();
 
-#if !defined(USE_UNSAFE_FILE_SEARCH)
+#if !defined(LOADER_USE_UNSAFE_FILE_SEARCH)
     ASSERT_FALSE(log.find("Loader is using non-secure environment variable lookup for"));
 #else
     ASSERT_TRUE(log.find("Loader is using non-secure environment variable lookup for"));
@@ -174,40 +204,38 @@ TEST(EnvVarICDOverrideSetup, XDG) {
     // so that the test app can find them.  Include some badly specified elements as well.
     // Need to redirect the 'home' directory
     fs::path HOME = "/home/fake_home";
-    set_env_var("HOME", HOME.str());
-    set_env_var("XDG_CONFIG_DIRS", ":/tmp/goober:::::/tmp/goober/::::");
-    set_env_var("XDG_CONFIG_HOME", ":/tmp/goober:::::/tmp/goober2/::::");
-    set_env_var("XDG_DATA_DIRS", "::::/tmp/goober3:/tmp/goober4/with spaces:::");
-    set_env_var("XDG_DATA_HOME", "::::/tmp/goober3:/tmp/goober4/with spaces:::");
+    EnvVarWrapper home_env_var{"HOME", HOME.str()};
+    EnvVarWrapper xdg_config_dirs_env_var{"XDG_CONFIG_DIRS", ":/tmp/goober:::::/tmp/goober/::::"};
+    EnvVarWrapper xdg_config_home_env_var{"XDG_CONFIG_HOME", ":/tmp/goober:::::/tmp/goober2/::::"};
+    EnvVarWrapper xdg_data_dirs_env_var{"XDG_DATA_DIRS", "::::/tmp/goober3:/tmp/goober4/with spaces:::"};
+    EnvVarWrapper xdg_data_home_env_var{"XDG_DATA_HOME", "::::/tmp/goober3:/tmp/goober4/with spaces:::"};
 
     FrameworkEnvironment env{};
-    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA));
-    env.get_test_icd().physical_devices.push_back({});
+    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA)).add_physical_device("physical_device_0");
 
     InstWrapper inst{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst.create_info, env.debug_log);
     inst.CheckCreate();
 
-    auto check_paths = [](DebugUtilsLogger const& debug_log, ManifestCategory category, fs::path const& HOME) {
+    auto check_paths = [](DebugUtilsLogger const& debug_log, ManifestCategory category) {
         EXPECT_TRUE(debug_log.find((fs::path("/tmp/goober/vulkan") / category_path_name(category)).str()));
         EXPECT_TRUE(debug_log.find((fs::path("/tmp/goober2/vulkan") / category_path_name(category)).str()));
         EXPECT_TRUE(debug_log.find((fs::path("/tmp/goober3/vulkan") / category_path_name(category)).str()));
         EXPECT_TRUE(debug_log.find((fs::path("/tmp/goober4/with spaces/vulkan") / category_path_name(category)).str()));
     };
-    check_paths(env.debug_log, ManifestCategory::icd, HOME);
-    check_paths(env.debug_log, ManifestCategory::implicit_layer, HOME);
-    check_paths(env.debug_log, ManifestCategory::explicit_layer, HOME);
+    check_paths(env.debug_log, ManifestCategory::icd);
+    check_paths(env.debug_log, ManifestCategory::implicit_layer);
+    check_paths(env.debug_log, ManifestCategory::explicit_layer);
 }
 // Check that a json file in the paths don't cause the loader to crash
 TEST(EnvVarICDOverrideSetup, XDGContainsJsonFile) {
     // Set up a layer path that includes default and user-specified locations,
     // so that the test app can find them.  Include some badly specified elements as well.
     // Need to redirect the 'home' directory
-    set_env_var("XDG_CONFIG_DIRS", "bad_file.json");
+    EnvVarWrapper xdg_config_dirs_env_var{"XDG_CONFIG_DIRS", "bad_file.json"};
 
     FrameworkEnvironment env{};
-    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA));
-    env.get_test_icd().physical_devices.push_back({});
+    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA)).add_physical_device("physical_device_0");
 
     InstWrapper inst{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst.create_info, env.debug_log);
@@ -247,11 +275,10 @@ TEST(EnvVarICDOverrideSetup, TestBothDriverEnvVars) {
 
     // Add a driver that isn't enabled with the environment variable
     env.add_icd(TestICDDetails(TEST_ICD_PATH_EXPORT_NONE).set_discovery_type(ManifestDiscoveryType::env_var));
-    env.get_test_icd(0).physical_devices.emplace_back("pd0");
-    env.get_test_icd(0).physical_devices.emplace_back("pd1");
+    env.get_test_icd(0).add_physical_device("pd0").add_physical_device("pd1");
 
     env.add_icd(TestICDDetails(TEST_ICD_PATH_EXPORT_NONE).set_discovery_type(ManifestDiscoveryType::add_env_var));
-    env.get_test_icd(0).physical_devices.emplace_back("pd2");
+    env.get_test_icd(0).add_physical_device("pd2");
 
     InstWrapper inst{env.vulkan_functions};
     inst.CheckCreate();
@@ -263,12 +290,11 @@ TEST(EnvVarICDOverrideSetup, TestBothDriverEnvVars) {
     ASSERT_EQ(phys_dev_count, 3U);
 }
 
-#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__)
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__GNU__)
 // Test VK_LAYER_PATH environment variable
 TEST(EnvVarICDOverrideSetup, TestOnlyLayerEnvVar) {
     FrameworkEnvironment env{};
-    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA));
-    env.get_test_icd().physical_devices.push_back({});
+    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA)).add_physical_device("physical_device_0");
     env.platform_shim->redirect_path("/tmp/carol", env.get_folder(ManifestLocation::explicit_layer_env_var).location());
 
     const char* layer_name = "TestLayer";
@@ -282,11 +308,10 @@ TEST(EnvVarICDOverrideSetup, TestOnlyLayerEnvVar) {
     // so that the test app can find them.  Include some badly specified elements as well.
     // Need to redirect the 'home' directory
     fs::path HOME = "/home/fake_home";
-    set_env_var("HOME", HOME.str());
+    EnvVarWrapper home_env_var{"HOME", HOME.str()};
     std::string vk_layer_path = ":/tmp/carol::::/:";
     vk_layer_path += (HOME / "/ with spaces/:::::/tandy:").str();
-    set_env_var("VK_LAYER_PATH", vk_layer_path);
-    EnvVarCleaner layer_path_cleaner("VK_LAYER_PATH");
+    EnvVarWrapper layer_path_env_var{"VK_LAYER_PATH", vk_layer_path};
     InstWrapper inst1{env.vulkan_functions};
     inst1.create_info.add_layer(layer_name);
     FillDebugUtilsCreateDetails(inst1.create_info, env.debug_log);
@@ -314,8 +339,7 @@ TEST(EnvVarICDOverrideSetup, TestOnlyLayerEnvVar) {
 // Test VK_ADD_LAYER_PATH environment variable
 TEST(EnvVarICDOverrideSetup, TestOnlyAddLayerEnvVar) {
     FrameworkEnvironment env{};
-    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA));
-    env.get_test_icd().physical_devices.push_back({});
+    env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_2_EXPORT_ICD_GPDPA)).add_physical_device("physical_device_0");
     env.platform_shim->redirect_path("/tmp/carol", env.get_folder(ManifestLocation::explicit_layer_add_env_var).location());
 
     const char* layer_name = "TestLayer";
@@ -329,11 +353,10 @@ TEST(EnvVarICDOverrideSetup, TestOnlyAddLayerEnvVar) {
     // so that the test app can find them.  Include some badly specified elements as well.
     // Need to redirect the 'home' directory
     fs::path HOME = "/home/fake_home";
-    set_env_var("HOME", HOME.str());
+    EnvVarWrapper home_env_var{"HOME", HOME.str()};
     std::string vk_layer_path = ":/tmp/carol::::/:";
     vk_layer_path += (HOME / "/ with spaces/:::::/tandy:").str();
-    set_env_var("VK_ADD_LAYER_PATH", vk_layer_path);
-    EnvVarCleaner add_layer_path_cleaner("VK_ADD_LAYER_PATH");
+    EnvVarWrapper add_layer_path_env_var{"VK_ADD_LAYER_PATH", vk_layer_path};
 
     InstWrapper inst1{env.vulkan_functions};
     inst1.create_info.add_layer(layer_name);
@@ -364,7 +387,7 @@ TEST(EnvVarICDOverrideSetup, TestOnlyAddLayerEnvVar) {
 // Test that the driver filter select will only enable driver manifest files that match the filter
 TEST(EnvVarICDOverrideSetup, FilterSelectDriver) {
     FrameworkEnvironment env{};
-    const char* filter_select_env_var = "VK_LOADER_DRIVERS_SELECT";
+    EnvVarWrapper filter_select_env_var{"VK_LOADER_DRIVERS_SELECT"};
 
     env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_6).set_disable_icd_inc(true).set_json_name("ABC_ICD"));
     env.add_icd(TestICDDetails{TEST_ICD_PATH_VERSION_6, VK_API_VERSION_1_2}.set_disable_icd_inc(true).set_json_name("BCD_ICD"));
@@ -386,7 +409,7 @@ TEST(EnvVarICDOverrideSetup, FilterSelectDriver) {
 
     // Match full-name
     env.debug_log.clear();
-    set_env_var(filter_select_env_var, "ABC_ICD.json");
+    filter_select_env_var.set_new_value("ABC_ICD.json");
 
     InstWrapper inst2{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst2.create_info, env.debug_log);
@@ -404,7 +427,7 @@ TEST(EnvVarICDOverrideSetup, FilterSelectDriver) {
 
     // Match prefix
     env.debug_log.clear();
-    set_env_var(filter_select_env_var, "ABC*");
+    filter_select_env_var.set_new_value("ABC*");
 
     InstWrapper inst3{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst3.create_info, env.debug_log);
@@ -422,7 +445,7 @@ TEST(EnvVarICDOverrideSetup, FilterSelectDriver) {
 
     // Match suffix
     env.debug_log.clear();
-    set_env_var(filter_select_env_var, "*C_ICD.json");
+    filter_select_env_var.set_new_value("*C_ICD.json");
 
     InstWrapper inst4{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst4.create_info, env.debug_log);
@@ -440,7 +463,7 @@ TEST(EnvVarICDOverrideSetup, FilterSelectDriver) {
 
     // Match sub-string
     env.debug_log.clear();
-    set_env_var(filter_select_env_var, "*BC*");
+    filter_select_env_var.set_new_value("*BC*");
 
     InstWrapper inst5{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst5.create_info, env.debug_log);
@@ -458,7 +481,7 @@ TEST(EnvVarICDOverrideSetup, FilterSelectDriver) {
 
     // Match all with star '*'
     env.debug_log.clear();
-    set_env_var(filter_select_env_var, "*");
+    filter_select_env_var.set_new_value("*");
 
     InstWrapper inst6{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst6.create_info, env.debug_log);
@@ -476,7 +499,7 @@ TEST(EnvVarICDOverrideSetup, FilterSelectDriver) {
 
     // Match all with special name
     env.debug_log.clear();
-    set_env_var(filter_select_env_var, "~all~");
+    filter_select_env_var.set_new_value("~all~");
 
     InstWrapper inst7{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst7.create_info, env.debug_log);
@@ -491,13 +514,30 @@ TEST(EnvVarICDOverrideSetup, FilterSelectDriver) {
     ASSERT_TRUE(env.debug_log.find_prefix_then_postfix("Found ICD manifest file", "CDE_ICD.json"));
     ASSERT_FALSE(env.debug_log.find_prefix_then_postfix("CDE_ICD.json", "ignored because not selected by env var"));
     ASSERT_FALSE(env.debug_log.find_prefix_then_postfix("CDE_ICD.json", "ignored because it was disabled by env var"));
+
+    // The full-name string is not a valid match if it doesn't also include the file extension
+    env.debug_log.clear();
+    filter_select_env_var.set_new_value("ABC_ICD");
+
+    InstWrapper inst8{env.vulkan_functions};
+    FillDebugUtilsCreateDetails(inst8.create_info, env.debug_log);
+    inst8.CheckCreate(VK_ERROR_INCOMPATIBLE_DRIVER);
+
+    ASSERT_TRUE(env.debug_log.find_prefix_then_postfix("Found ICD manifest file", "ABC_ICD.json"));
+    ASSERT_TRUE(env.debug_log.find_prefix_then_postfix("ABC_ICD.json", "ignored because not selected by env var"));
+    ASSERT_FALSE(env.debug_log.find_prefix_then_postfix("ABC_ICD.json", "ignored because it was disabled by env var"));
+    ASSERT_TRUE(env.debug_log.find_prefix_then_postfix("Found ICD manifest file", "BCD_ICD.json"));
+    ASSERT_TRUE(env.debug_log.find_prefix_then_postfix("BCD_ICD.json", "ignored because not selected by env var"));
+    ASSERT_FALSE(env.debug_log.find_prefix_then_postfix("BCD_ICD.json", "ignored because it was disabled by env var"));
+    ASSERT_TRUE(env.debug_log.find_prefix_then_postfix("Found ICD manifest file", "CDE_ICD.json"));
+    ASSERT_TRUE(env.debug_log.find_prefix_then_postfix("CDE_ICD.json", "ignored because not selected by env var"));
+    ASSERT_FALSE(env.debug_log.find_prefix_then_postfix("CDE_ICD.json", "ignored because it was disabled by env var"));
 }
 
 // Test that the driver filter disable disables driver manifest files that match the filter
 TEST(EnvVarICDOverrideSetup, FilterDisableDriver) {
     FrameworkEnvironment env{};
-    const char* filter_disable_env_var = "VK_LOADER_DRIVERS_DISABLE";
-    EnvVarCleaner filter_disable_cleaner(filter_disable_env_var);
+    EnvVarWrapper filter_disable_env_var{"VK_LOADER_DRIVERS_DISABLE"};
 
     env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_6).set_disable_icd_inc(true).set_json_name("ABC_ICD"));
     env.add_icd(TestICDDetails{TEST_ICD_PATH_VERSION_6, VK_API_VERSION_1_2}.set_disable_icd_inc(true).set_json_name("BCD_ICD"));
@@ -519,7 +559,7 @@ TEST(EnvVarICDOverrideSetup, FilterDisableDriver) {
 
     // Match full-name
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "ABC_ICD.json");
+    filter_disable_env_var.set_new_value("ABC_ICD.json");
 
     InstWrapper inst2{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst2.create_info, env.debug_log);
@@ -537,7 +577,7 @@ TEST(EnvVarICDOverrideSetup, FilterDisableDriver) {
 
     // Match prefix
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "ABC_*");
+    filter_disable_env_var.set_new_value("ABC_*");
 
     InstWrapper inst3{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst3.create_info, env.debug_log);
@@ -555,7 +595,7 @@ TEST(EnvVarICDOverrideSetup, FilterDisableDriver) {
 
     // Match suffix
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "*C_ICD.json");
+    filter_disable_env_var.set_new_value("*C_ICD.json");
 
     InstWrapper inst4{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst4.create_info, env.debug_log);
@@ -573,7 +613,7 @@ TEST(EnvVarICDOverrideSetup, FilterDisableDriver) {
 
     // Match substring
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "*BC*");
+    filter_disable_env_var.set_new_value("*BC*");
 
     InstWrapper inst5{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst5.create_info, env.debug_log);
@@ -591,7 +631,7 @@ TEST(EnvVarICDOverrideSetup, FilterDisableDriver) {
 
     // Match all with star '*'
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "*");
+    filter_disable_env_var.set_new_value("*");
 
     InstWrapper inst6{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst6.create_info, env.debug_log);
@@ -609,7 +649,7 @@ TEST(EnvVarICDOverrideSetup, FilterDisableDriver) {
 
     // Match all with special name
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "~all~");
+    filter_disable_env_var.set_new_value("~all~");
 
     InstWrapper inst7{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst7.create_info, env.debug_log);
@@ -630,10 +670,8 @@ TEST(EnvVarICDOverrideSetup, FilterDisableDriver) {
 // appropriate drivers are enabled and disabled
 TEST(EnvVarICDOverrideSetup, FilterSelectAndDisableDriver) {
     FrameworkEnvironment env{};
-    const char* filter_select_env_var = "VK_LOADER_DRIVERS_SELECT";
-    const char* filter_disable_env_var = "VK_LOADER_DRIVERS_DISABLE";
-    EnvVarCleaner filter_select_cleaner(filter_select_env_var);
-    EnvVarCleaner filter_disable_cleaner(filter_disable_env_var);
+    EnvVarWrapper filter_disable_env_var{"VK_LOADER_DRIVERS_DISABLE"};
+    EnvVarWrapper filter_select_env_var{"VK_LOADER_DRIVERS_SELECT"};
 
     env.add_icd(TestICDDetails(TEST_ICD_PATH_VERSION_6).set_disable_icd_inc(true).set_json_name("ABC_ICD"));
     env.add_icd(TestICDDetails{TEST_ICD_PATH_VERSION_6, VK_API_VERSION_1_2}.set_disable_icd_inc(true).set_json_name("BCD_ICD"));
@@ -655,8 +693,8 @@ TEST(EnvVarICDOverrideSetup, FilterSelectAndDisableDriver) {
 
     // Disable two, but enable one
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "*BC*");
-    set_env_var(filter_select_env_var, "BCD*");
+    filter_disable_env_var.set_new_value("*BC*");
+    filter_select_env_var.set_new_value("BCD*");
 
     InstWrapper inst2{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst2.create_info, env.debug_log);
@@ -674,8 +712,8 @@ TEST(EnvVarICDOverrideSetup, FilterSelectAndDisableDriver) {
 
     // Disable all, but enable two
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "*");
-    set_env_var(filter_select_env_var, "*BC*");
+    filter_disable_env_var.set_new_value("*");
+    filter_select_env_var.set_new_value("*BC*");
 
     InstWrapper inst3{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst3.create_info, env.debug_log);
@@ -693,8 +731,8 @@ TEST(EnvVarICDOverrideSetup, FilterSelectAndDisableDriver) {
 
     // Disable all, but enable all
     env.debug_log.clear();
-    set_env_var(filter_disable_env_var, "*");
-    set_env_var(filter_select_env_var, "*");
+    filter_disable_env_var.set_new_value("*");
+    filter_select_env_var.set_new_value("*");
 
     InstWrapper inst4{env.vulkan_functions};
     FillDebugUtilsCreateDetails(inst4.create_info, env.debug_log);
